/*
 * Copyright (c) 2004 by Tsuyoshi Iguchi (zinnia@risky-safety.org)
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL THE AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 *
 *	$Id: misc.c,v 1.5 2004/05/18 03:12:20 zinnia Exp $
 */

#include "sps.h"

static GHashTable* context_hash = NULL;

SPS_Render_Context* SPS_Render_Context_Lookup(lua_State* state)
{
	return g_hash_table_lookup(context_hash, state);
}

void SPS_Render_Context_Register(SPS_Render_Context* context, lua_State* state)
{
	gpointer oldvalue, oldkey;
	if(context_hash == NULL){
		context_hash = g_hash_table_new(g_direct_hash, g_direct_equal);
	}

	if(g_hash_table_lookup_extended(context_hash, state, &oldvalue, &oldkey)){
		g_hash_table_remove(context_hash, state);
	}
	g_hash_table_insert(context_hash, state, context);
}

/* ---------------------------------------- */
/* utility functions */

static struct _coltbl {
	const char* name;
	Uint8 r;
	Uint8 g;
	Uint8 b;
} colortbl[] = {
	{"black", 0, 0, 0},
	{"white", 0xff, 0xff, 0xff},
	{"red", 0xff, 0, 0},
	{"green", 0, 0xff, 0},
	{"blue", 0, 0, 0xff},
	{"cyan", 0, 0xff, 0xff},
	{"yellow", 0xff, 0xff, 0},
	{"magenta", 0xff, 0, 0xff},
	{NULL, 0, 0, 0},
};


static Uint8 GetNibble(char c)
{
	char p = tolower(c);
	if(p >= '0' && p <= '9'){
		return p - '0';
	} else if(p >= 'a' && p <= 'f') {
		return p - 'a' + 10;
	}
	return 0;
}

static Uint8 GetOctet(const char* expression, int offset)
{
	Uint8 result;
	result = GetNibble(*(expression + offset));
	result <<= 4;
	result |= GetNibble(*(expression + offset + 1)) & 0xf;

	return result;
}

Uint32 ParseColor(SPS_Render_Context* context, const char* text)
{
	int i; Uint8 r, g, b; int len;
	char tmp[7];

	for(i = 0; colortbl[i].name != NULL; i++){
		if(!strcasecmp(text, colortbl[i].name)) return SDL_MapRGB(display->format, colortbl[i].r, colortbl[i].g, colortbl[i].b);

	}
	len = strlen(text);
	if(len > 6){
		strncpy(tmp, text, 6);
		tmp[6] = '\0';
	}
	else {
		int t;
		for(t = 0; t < 6; t++) tmp[t] = '0';
		strcpy(tmp + (6 - strlen(text)), text);
	}
	r = GetOctet(tmp, 0);
	g = GetOctet(tmp, 2);
	b = GetOctet(tmp, 4);

	fprintf(stderr, "%s -> %s -> %02x, %02x, %02x\n", text, tmp, r, g, b);
	
	return SDL_MapRGB(display->format, r, g, b);
}

SPS_VERTICAL_ALIGN SPS_Vertical_Align_Parse(const char* expression)
{
	if(!strcasecmp(expression, "top")) return SPS_VERTICAL_ALIGN_TOP;
	if(!strcasecmp(expression, "bottom")) return SPS_VERTICAL_ALIGN_BOTTOM;
	if(!strcasecmp(expression, "center")) return SPS_VERTICAL_ALIGN_CENTER;

	return SPS_VERTICAL_ALIGN_BOTTOM;
}
		
SPS_HORIZONTAL_ALIGN SPS_Horizontal_Align_Parse(const char* expression)
{
	if(!strcasecmp(expression, "right")) return SPS_HORIZONTAL_ALIGN_RIGHT;
	if(!strcasecmp(expression, "left")) return SPS_HORIZONTAL_ALIGN_LEFT;
	if(!strcasecmp(expression, "center")) return SPS_HORIZONTAL_ALIGN_CENTER;

	return SPS_HORIZONTAL_ALIGN_LEFT;
}

SPS_BGSTYLE SPS_BgStyle_Parse(const char* expression)
{
	if(!strcasecmp(expression, "tile")) return SPS_BGSTYLE_TILE;
	if(!strcasecmp(expression, "zoom")) return SPS_BGSTYLE_ZOOM;
	if(!strcasecmp(expression, "center")) return SPS_BGSTYLE_CENTER;
}

static struct _ttfstyle {
	int character;
	int style;
} ttfstyle[] = {
	{'b', TTF_STYLE_BOLD},
	{'B', TTF_STYLE_BOLD},
	{'i', TTF_STYLE_ITALIC},
	{'I', TTF_STYLE_ITALIC},
	{'u', TTF_STYLE_UNDERLINE},
	{'U', TTF_STYLE_UNDERLINE},
	{0, 0},
};

int TTF_Style_Parse(const char* expression)
{
	int result = 0;
	int i;
	for(i = 0; ttfstyle[i].character != 0; i++)
	{
		if(strchr(expression, ttfstyle[i].character)){
			result |= ttfstyle[i].style;
		}
	}
	return result;
}

int RatioToPixelH(SPS_Render_Context* context, double h)
{
	int result = display->w * h / 100;
	return result;
}

int RatioToPixelV(SPS_Render_Context* context, double v)
{
	int result = display->h * v / 100;
	return result;
}

int RatioToPixel(SPS_Render_Context* context, double l)
{
	if(display->w > display->h){
		return RatioToPixelV(context, l);
	} else {
		return RatioToPixelH(context, l);
	}
}

gchar* get_utf8_string(const char* str)
{
	gchar* result = g_convert(str, strlen(str), "UTF-8", DEFAULT_ENCODING, 
			 NULL, NULL, NULL);
	return result;
}

/* -------------------- */

TTF_Font* GetFont(SPS_Render_Context* context, const char* fontindex)
{
	return g_hash_table_lookup(context->fonts, fontindex);
}
SPS_Sprite* GetIcon(SPS_Render_Context* context, const char* iconindex)
{
	return g_hash_table_lookup(context->icons, iconindex);
}
void SDL_UnMapRGB(SDL_PixelFormat* fmt, SDL_Color* dst, Uint32 color)
{
	Uint8 r, g, b;
	SDL_GetRGB(color, fmt, &r, &g, &b);
	dst->r = r;
	dst->g = g;
	dst->b = b;
}


void usage()
{
	fprintf(stderr, "usage: i don't know. \n");
	exit(1);
}


#ifdef USE_OPENGL
void setup_opengl()
{
    /* $B%7%'!<%G%#%s%0%b%G%k$O(B Gouraud ($B$J$a$i$+(B) */
    glShadeModel( GL_SMOOTH );

    /* $BN"LL$r<h$j=|$/(B */
    glCullFace( GL_BACK );
    glFrontFace( GL_CCW );
    glEnable( GL_CULL_FACE );

    /* $B>C5n;~$N?'$r%;%C%H(B */
//    glClearColor( 0.1, 0.1, 0.2, 0 );
    glClearColor( 0, 0, 0, 0 );

    /* $B%S%e!<%]!<%H$r@_Dj(B */
    glViewport( 0, 0, display->w, display->h);

}

void SDL_GL_Enter2DMode(gboolean origin_center)
{
	/* Note, there may be other things you need to change,
	   depending on how you have your OpenGL state set up.
	*/
	glPushAttrib(GL_ENABLE_BIT);
	glDisable(GL_DEPTH_TEST);
	glDisable(GL_CULL_FACE);
	glEnable(GL_TEXTURE_2D);

	/* This allows alpha blending of 2D textures with the scene */
	glEnable(GL_BLEND);
	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);

	glViewport(0, 0, display->w, display->h);

	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	if(origin_center){
		glOrtho(-(GLdouble)display->w / 2, (GLdouble)display->w / 2, (GLdouble)display->h / 2, -(GLdouble)display->h / 2, 0.0, 1.0);
		
	} else {
		glOrtho(0.0, (GLdouble)display->w, (GLdouble)display->h, 0.0, 0.0, 1.0);
	}

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	glTexEnvf(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
}

void SDL_GL_Leave2DMode()
{
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();

	glMatrixMode(GL_PROJECTION);
	glPopMatrix();

	glPopAttrib();
}

#endif

#define READLINES_LINE_MAX 0x1000
GList* ReadLines(const char* path)
{
	FILE* fp;
	GString* linetmp;
	char line[READLINES_LINE_MAX + 1];
	GList* result = NULL;

	fp = fopen(path, "r");
	if(fp == NULL) return result;

	while(fgets(line, READLINES_LINE_MAX, fp)){
		linetmp = g_string_new(line);
		result = g_list_append(result, linetmp);
	}
	fclose(fp);
	return result;
}

